/* 
** Saves and restores the state of xfpovray.
** State is stored in $HOME/.xfpovray/xfpovray.state
*/


#include "forms.h"
#include <stdlib.h>
#include "xfpovray.h"
#include "pov-defaults.h"

extern char *xfp_version;
extern int auto_view;
extern struct pov_options pov;

extern char *CONFIG_POV_COMMAND;
extern char *CONFIG_POV_LIB_PATH;
extern char *CONFIG_POV_EDITOR;
extern char *CONFIG_DISPLAY_COMMAND;
extern char *CONFIG_POV_DIR;
extern char *CONFIG_SCENE_FILTER;
extern char *CONFIG_IMAGE_FILTER;
extern char *CONFIG_POV_WIDTH; 
extern char *CONFIG_POV_HEIGHT;

extern XFDATA_S *xfData;
extern PARSING_PAGE_S   *parsingPage;
extern OUTPUT_PAGE_S	*outputPage;
extern TRACING_PAGE_S   *tracingPage;
extern ANIMATION_PAGE_S *animationPage;
extern CONFIG_PAGE_S	*configPage;


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void save_state (void)
{
    FILE *state_file;
    char filename[256];
    char temp[256];
    int i;
   
       
    /* Get home directory, and set filepath */
    sprintf (filename, "%s/.xfpovray/xfpovray.state", getenv("HOME"));

    if ((state_file = fopen(filename, "w")) == NULL) {
       fprintf (stderr, "Unable to open %s for writing! State not saved.\n",
               filename);
       return;
    }


    /* Write the current state */
    sprintf(temp, "/* xfpovray %s, generated by state.c */\n", xfp_version); 
    fprintf (state_file, temp);

    /* Parsing */
    
    for (i=0; i<5; i++) {
        if (strcmp(fl_get_input (parsingPage->libpathInput[i]), "") != 0)
           fprintf (state_file, "%s\t/* library path */\n", 
                   fl_get_input (parsingPage->libpathInput[i]));    
        else
           fprintf (state_file, "NULL\t/* library path */\n", 
                   fl_get_input (parsingPage->libpathInput[i]));    
    }
    
    fprintf (state_file, "%d\t/* language version */\n", pov.version);
    fprintf (state_file, "%d\t/* remove bounds */\n", pov.remove_bounds);
    fprintf (state_file, "%d\t/* split unions */\n", pov.split_unions);
    

    /* Output */

    if (strcmp(fl_get_input(outputPage->imageHeightInput), "") == 0)
       fprintf (state_file, "NULL\t/* image height */\n");
    else
       fprintf (state_file, "%d\t/* image_height */\n", pov.height);
 
    if (strcmp(fl_get_input(outputPage->imageWidthInput), "") == 0)
       fprintf (state_file, "NULL\t/* image width */\n");
    else
       fprintf (state_file, "%d\t/* image_width */\n", pov.width);
  
    fprintf (state_file, "%d\t/* display_during_render */\n", 
        pov.display_during_render);
    fprintf (state_file, "%d\t/* output to file */\n", pov.output_to_file);
    fprintf (state_file, "%d\t/* verbose */\n", pov.verbose);
    fprintf (state_file, "%d\t/* palette */\n", pov.palette);
    fprintf (state_file, "%d\t/* output filetype */\n", pov.output_filetype);


    /* Tracing */ 

    fprintf (state_file, "%d\t/* quality */\n", pov.quality);
    fprintf (state_file, "%d\t/* aa depth */\n", pov.aa_depth);
    fprintf (state_file, "%d\t/* aa */\n", pov.antialias);
    fprintf (state_file, "%d\t/* aa method */\n", pov.aa_sampling_method);
    fprintf (state_file, "%d\t/* radiosity */\n", pov.radiosity);
    fprintf (state_file, "%d\t/* bounding */\n", pov.bounding);
    fprintf (state_file, "%d\t/* light buffer */\n", pov.light_buffer);
    fprintf (state_file, "%d\t/* vista buffer */\n", pov.vista_buffer);
    fprintf (state_file, "%d\t/* jitter */\n", pov.jitter);

    if (strcmp(fl_get_input (tracingPage->antialiasThreshold), "") == 0)
       fprintf (state_file, "NULL\t/* aa threshold */\n");
    else
       fprintf (state_file, "%s\t/* aa threshold */\n", 
           fl_get_input (tracingPage->antialiasThreshold));
  
    if (strcmp(fl_get_input (tracingPage->boundingThreshold), "") == 0)
       fprintf (state_file, "NULL\t/* bounding threshold */\n");
    else
       fprintf (state_file, "%s\t/* bounding threshold */\n", 
           fl_get_input (tracingPage->boundingThreshold));
  
    if (strcmp(fl_get_input (tracingPage->jitterInput), "") == 0)
       fprintf (state_file, "NULL\t/* jitter amount */\n");
    else
       fprintf (state_file, "%s\t/* jitter amount */\n", 
           fl_get_input (tracingPage->jitterInput));
 
    /* Animation */ 

    if (strcmp(fl_get_input (animationPage->frameStartInput), "") == 0)
       fprintf (state_file, "%d\t/* Initial frame */\n", POV_INITIAL_FRAME);
    else
       fprintf (state_file, "%s\t/* Initial frame */\n",
           fl_get_input (animationPage->frameStartInput));

    if (strcmp(fl_get_input (animationPage->frameEndInput), "") == 0)
       fprintf(state_file, "%d\t/* Final frame */\n", POV_FINAL_FRAME);
    else
       fprintf (state_file, "%s\t/* Final frame */\n", 
           fl_get_input (animationPage->frameEndInput));

    if (strcmp(fl_get_input (animationPage->subsetFrameStartInput), "") == 0)
       fprintf (state_file, "NULL\t/* Subset initial frame */\n");
    else
       fprintf (state_file, "%s\t/* Subset initial frame */\n",
           fl_get_input (animationPage->subsetFrameStartInput));

    if (strcmp(fl_get_input (animationPage->subsetFrameEndInput), "") == 0)
       fprintf (state_file, "NULL\t/* Subset end frame */\n");
    else
       fprintf (state_file, "%s\t/* Subset end frame */\n",
           fl_get_input (animationPage->subsetFrameEndInput));


    if (strcmp(fl_get_input (animationPage->clockStartInput), "") == 0)
       fprintf (state_file, "%f\t/* Initial clock */\n", POV_INITIAL_CLOCK);
    else
       fprintf (state_file, "%s\t/* Initial clock */\n",
           fl_get_input (animationPage->clockStartInput));

    if (strcmp(fl_get_input (animationPage->clockEndInput), "") == 0)
       fprintf (state_file, "%f\t/* Final clock */\n", POV_FINAL_CLOCK);
    else
       fprintf (state_file, "%s\t/* Final clock */\n", 
           fl_get_input (animationPage->clockEndInput));

    fprintf (state_file, "%d\t/* Cyclic animation */\n", pov.cyclic_animation);
    fprintf (state_file, "%d\t/* Field rendering */\n", pov.field_rendering);

    if (strcmp(fl_get_input (animationPage->clockExternalInput), "") == 0)
       fprintf (state_file, "%f\t/* External clock */\n", POV_EXTERNAL_CLOCK);
    else
       fprintf (state_file, "%s\t/* External clock */\n", 
           fl_get_input (animationPage->clockExternalInput));


    /* Configuration */ 

    fprintf (state_file, "%s\t/* configuration */\n", CONFIG_POV_COMMAND);
    fprintf (state_file, "%s\t/* configuration */\n", CONFIG_POV_LIB_PATH);
    fprintf (state_file, "%s\t/* configuration */\n", CONFIG_POV_EDITOR);
    fprintf (state_file, "%s\t/* configuration */\n", CONFIG_DISPLAY_COMMAND);
    fprintf (state_file, "%s\t/* configuration */\n", CONFIG_POV_DIR);
    fprintf (state_file, "%s\t/* configuration */\n", CONFIG_SCENE_FILTER);
    fprintf (state_file, "%s\t/* configuration */\n", CONFIG_IMAGE_FILTER);
    fprintf (state_file, "%s\t/* configuration */\n", CONFIG_POV_WIDTH); 
    fprintf (state_file, "%s\t/* configuration */\n", CONFIG_POV_HEIGHT);

    /* Auto image viewer */
    fprintf (state_file, "%d\t/* auto_view */\n", auto_view);

    /* File buffer */
    fprintf (state_file, "%d\t/* buffer output */\n", pov.buffer_output);
    fprintf (state_file, "%d\t/* buffer size */\n", pov.buffer_size);

    fclose (state_file);

/*    
    flx_set_dialog_font (FL_BOLD_STYLE, FL_NORMAL_SIZE);
    flx_show_dialog ("message", "xfpovray state saved");
*/
 
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void get_state (int show_warning)
{
    FILE *state_file;
    char filename[512];
    int i;
    char buf[80];
    char *value;
    const char delimiter[] = "\t";     
    
       
    /* Get home directory, and set filepath */
    sprintf(filename, "%s/.xfpovray/xfpovray.state", getenv("HOME"));

    /* If no state file exists, show a warning, or do nothing */
    if ((state_file = fopen(filename, "r")) == NULL) {
       if (show_warning) {
          flx_set_dialog_font (FL_BOLD_STYLE, FL_NORMAL_SIZE);
          flx_show_dialog ("info", "No state file found!\n");
       }
       return;
    }

    /* Read the header.  Check for version consistency. */
    fgets (buf, sizeof(buf), state_file);
    if (strstr(buf, xfp_version) == NULL) {
       flx_set_dialog_font (FL_BOLD_STYLE, FL_NORMAL_SIZE);
       flx_show_dialog ("warning", 
       "\nObsolete state file.\nYou will have to reconfigure\nthe state of xfpovray\nand resave.");
       save_state();
    }
         
    /* 
    ** Read the current state 
    ** Geeze, is this ugly...
    ** TODO: Improve state file handling
    */
    i = 1;
    while (fgets(buf, sizeof(buf), state_file) != NULL) {

          value = strtok(buf, delimiter);
  
          switch (i) {
                 case 1: if (strcmp(value, "NULL") != 0)
                            fl_set_input(parsingPage->libpathInput[i-1], value);
                         break;
                 case 2: if (strcmp(value, "NULL") != 0)
                            fl_set_input(parsingPage->libpathInput[i-1], value);
                         break;
                 case 3: if (strcmp(value, "NULL") != 0)
                            fl_set_input(parsingPage->libpathInput[i-1], value);
                         break;
                 case 4: if (strcmp(value, "NULL") != 0)
                            fl_set_input(parsingPage->libpathInput[i-1], value);
                         break;
                 case 5: if (strcmp(value, "NULL") != 0)
                            fl_set_input(parsingPage->libpathInput[i-1], value);
                         break;
                 case 6: pov.version = atoi(value); 
                         fl_set_choice (parsingPage->languageVersion, 
			     pov.version);
                         break;
                 case 7: pov.remove_bounds = atoi(value); 
                         fl_set_choice (parsingPage->removeBounds, 
			     pov.remove_bounds + 1);
                         break;
                 case 8: pov.split_unions = atoi(value); 
                         fl_set_choice (parsingPage->splitUnions, 
			     pov.split_unions + 1);
                         break;
                 case 9: pov.height = atoi(value); 
                         fl_set_input (outputPage->imageHeightInput, value);  
                         break;
                 case 10: pov.width = atoi(value); 
                         fl_set_input (outputPage->imageWidthInput, value);  
                         break;
                 case 11: pov.display_during_render = atoi(value); 
                         fl_set_choice (outputPage->displayDuringRender, 
			     pov.display_during_render + 1);
                         break;
                 case 12: pov.output_to_file = atoi(value); 
                         fl_set_choice (outputPage->imageToFile, 
			     pov.output_to_file + 1);
                         break;
                 case 13: pov.verbose = atoi(value); 
                         fl_set_choice (outputPage->verbosity, pov.verbose + 1);
                         break;                 
                 case 14: pov.palette = atoi(value); 
                         fl_set_choice (outputPage->palette, pov.palette);
                         break;
                 case 15: pov.output_filetype = atoi(value); 
                         fl_set_choice (outputPage->fileType, 
			     pov.output_filetype);
                         break;
                 case 16: pov.quality = atoi(value); 
                         fl_set_slider_value (tracingPage->qualitySlider, 
			     pov.quality);
                         break;
                 case 17: pov.aa_depth = atoi(value); 
                         fl_set_slider_value (tracingPage->antialiasSlider, 
			     pov.aa_depth);
                         break;
                 case 18: pov.antialias = atoi(value); 
                         fl_set_choice (tracingPage->antialias, 
			     pov.antialias + 1);
                         break;
                 case 19: pov.aa_sampling_method = atoi(value); 
                         fl_set_choice (tracingPage->antialiasMethod, 
			     pov.aa_sampling_method);
                         break;
                 case 20: pov.radiosity = atoi(value); 
                         fl_set_choice (tracingPage->radiosity, 
			     pov.radiosity + 1);
                         break;
                 case 21: pov.bounding = atoi(value); 
                         fl_set_choice (tracingPage->bounding, 
			     pov.bounding + 1);
                         break;
                 case 22: pov.light_buffer = atoi(value); 
                         fl_set_choice (tracingPage->lightBuffer, 
			     pov.light_buffer + 1);
                         break;
                 case 23: pov.vista_buffer = atoi(value); 
                         fl_set_choice (tracingPage->vistaBuffer, 
			     pov.vista_buffer + 1);
                         break;
                 case 24: pov.jitter = atoi(value); 
                         fl_set_choice (tracingPage->jitter, 
			     pov.jitter + 1);
                         break;
                 case 25: fl_set_input (tracingPage->antialiasThreshold, value);  
                         break;
                 case 26: fl_set_input (tracingPage->boundingThreshold, value);  
                         break;
                 case 27: fl_set_input (tracingPage->jitterInput, value);  
                         break;
 
                 /* Animation */ 

                 case 28: fl_set_input (animationPage->frameStartInput, value);  
                         break;
                 case 29: fl_set_input (animationPage->frameEndInput, value);  
                         break;
                 case 30: if (strcmp(value, "NULL") != 0)
		             fl_set_input 
			         (animationPage->subsetFrameStartInput, value);  
                          else
		             fl_set_input 
			         (animationPage->subsetFrameStartInput, "");  
                         break;
                 case 31: if (strcmp(value, "NULL") != 0)
		             fl_set_input 
			         (animationPage->subsetFrameEndInput,value);  
                          else
		             fl_set_input 
			         (animationPage->subsetFrameEndInput, "");  
			   
                         break;
                 case 32: fl_set_input (animationPage->clockStartInput, value);  
                         break;
                 case 33: fl_set_input (animationPage->clockEndInput, value);  
                         break;
                 case 34: pov.cyclic_animation = atoi(value); 
                         fl_set_choice (animationPage->cyclicAnimation, 
			     pov.cyclic_animation + 1);
                         break;
                 case 35: pov.field_rendering = atoi(value); 
                         fl_set_choice (animationPage->fieldRendering, 
			     pov.field_rendering + 1);
                         break;
                 case 36: fl_set_input (animationPage->clockExternalInput, 
		             value);  
                         break;


                 /* Configuration */ 
                 case 37: fl_set_input (xfData->configOptions[0], value);
                          CONFIG_POV_COMMAND = 
                              (char *) fl_get_input (xfData->configOptions[0]);
                          break;
                 case 38: fl_set_input (xfData->configOptions[1], value); 
                          CONFIG_POV_LIB_PATH = 
                              (char *) fl_get_input (xfData->configOptions[1]);
                          break;
                 case 39: fl_set_input (xfData->configOptions[2], value); 
                          CONFIG_POV_EDITOR =
                              (char *) fl_get_input (xfData->configOptions[2]);
                          break;
                 case 40: fl_set_input (xfData->configOptions[3], value); 
                          CONFIG_DISPLAY_COMMAND =
                              (char *) fl_get_input (xfData->configOptions[3]);
                          break;
                 case 41: fl_set_input (xfData->configOptions[4], value); 
                          CONFIG_POV_DIR =
                              (char *) fl_get_input (xfData->configOptions[4]);
                          break;
                 case 42: fl_set_input (xfData->configOptions[5], value); 
                          CONFIG_SCENE_FILTER =
                              (char *) fl_get_input (xfData->configOptions[5]);
                          break;
                 case 43: fl_set_input (xfData->configOptions[6], value); 
                          CONFIG_IMAGE_FILTER =
                              (char *) fl_get_input (xfData->configOptions[6]);
                          break;
                 case 44: fl_set_input (xfData->configOptions[7], value); 
                          CONFIG_POV_WIDTH =
                              (char *) fl_get_input (xfData->configOptions[7]);
                          break;
                 case 45: fl_set_input (xfData->configOptions[8], value); 
                          CONFIG_POV_HEIGHT =
                              (char *) fl_get_input (xfData->configOptions[8]);
                          break;
                 case 46: auto_view = atoi(value); 
                          fl_set_menu_item_mode (xfData->optionsMenu, 5, 
			      auto_view ? FL_PUP_CHECK : FL_PUP_BOX);
                          break;
                 
                 case 47: pov.buffer_output = atoi(value); 
                         fl_set_choice (outputPage->bufferOutput, 
			     pov.buffer_output + 1);
                         break;

                 case 48: pov.buffer_size = atoi(value); 
                         break;

		 default: break;
                 
          } /* endswitch */
                  
          i++;
          
    }  /* endwhile */     
     
    fclose (state_file);

 
}
